InspectFix = CreateFrame("Button", "InspectFixHiddenFrame", UIParent)

local BlizzardNotifyInspect = _G.NotifyInspect
local InspectPaperDollFrame_SetLevel = nil
local loaded = false

local lastunit
local function unitchange(self)
  local unit = self.unit
  if loaded and unit and unit ~= lastunit then
    lastunit = unit
    BlizzardNotifyInspect(unit)
    InspectFrame_UnitChanged(InspectFrame)
  end
end

local function inspectfilter(self, event, ...) 
  --myprint(event,...)
  if loaded then
    -- ignore an inspect target disappearance or change to non-player - ie, keep the window open
    if (event == "PLAYER_TARGET_CHANGED" or event == nil) and
       self.unit == "target" and
       (not UnitExists("target") or not UnitIsPlayer("target")) then
      return false
    end
  end
  return true
end
local function inspectonevent(self, event, ...)
  if inspectfilter(self, event, ...) then
    InspectFrame_OnEvent(self, event, ...)
  end
end
local function inspectonupdate(self)
  if inspectfilter(self, nil) then
    InspectFrame_OnUpdate(self)
  end
end

-- cache the inspect contents in case we lose our target (so GameTooltip:SetInventoryItem() no longer works)
local inspect_item = {}
local inspect_unit = nil
local function pdfupdate(self)
  if loaded then
    local id = self:GetID()
    local unit = InspectFrame.unit
    if unit and id then
      inspect_unit = unit
      inspect_item[id] = GetInventoryItemLink(unit, id)
    end
  end
end
local function pdfonenter(self)
  if loaded then
    local id = self:GetID()
    if id and inspect_item[id] and inspect_unit == InspectFrame.unit and
       GameTooltip:IsVisible() and GameTooltip:NumLines() == 1 then
      GameTooltip:SetOwner(self, "ANCHOR_RIGHT");
      GameTooltip:SetHyperlink(inspect_item[id])
    end
  end
end

local blockmsg = {}

-- prevent NotifyInspect interference from other addons
local function NIhook(unit)
  InspectFix:tryhook()
  if InspectFrame and InspectFrame:IsVisible() then
    local str = debugstack(2)
    --print(str)
    local addon = string.match(str,'[%s%c]+([^:%s%c]*)\\[^\\:%s%c]+:')
    addon = string.gsub(addon or "unknown",'I?n?t?e?r?f?a?c?e\\AddOns\\',"")
    if not string.find(str,"Blizzard_InspectUI") then
      blockmsg[addon] = blockmsg[addon] or {}
      local count = (blockmsg[addon].count or 0) + 1
      blockmsg[addon].count = count
      local now = GetTime()
      if not blockmsg[addon].lastwarn or (now - blockmsg[addon].lastwarn > 30) then -- throttle warnings
        print("InspectFix blocked a conflicting inspect request from "..addon.." ("..count.." occurences)")
        blockmsg[addon].lastwarn = now
      end
      return
    end
  end
  BlizzardNotifyInspect(unit)
end

-- prevent a lua error bug in pdf
local function setlevel_hook()
  if InspectFrame and InspectFrame.unit then
    return InspectPaperDollFrame_SetLevel()
  end
end

local hooked_uc = false
local hooked_pdf = false
local hookcnt = 0
function InspectFix:tryhook()
  if false and not hooked_uc and InspectFrame_UnitChanged then
    hooksecurefunc("InspectFrame_UnitChanged", unitchange)
    hooked_uc = true
  end

  if _G.InspectFrame_OnEvent then
    InspectFrame:SetScript("OnEvent", inspectonevent)
    hookcnt = hookcnt + 1
  end

  if _G.InspectFrame_OnUpdate then
    InspectFrame:SetScript("OnUpdate", inspectonupdate)
    hookcnt = hookcnt + 1
  end

  if _G.InspectPaperDollFrame_SetLevel and _G.InspectPaperDollFrame_SetLevel ~= setlevel_hook then
    InspectPaperDollFrame_SetLevel = _G.InspectPaperDollFrame_SetLevel
    _G.InspectPaperDollFrame_SetLevel = setlevel_hook
    hookcnt = hookcnt + 1
  end

  if not hooked_pdf and InspectPaperDollItemSlotButton_Update and InspectPaperDollItemSlotButton_OnEnter then
    hooksecurefunc("InspectPaperDollItemSlotButton_Update", pdfupdate)
    hooksecurefunc("InspectPaperDollItemSlotButton_OnEnter", pdfonenter)
    hooked_pdf = true
    hookcnt = hookcnt + 1
  end

  if hookcnt == 4 then
    hookcnt = hookcnt + 1
    print("InspectFix hook activated.")
  end
end
function InspectFix_OnEvent(event)
  if event == "ADDON_LOADED" then
    InspectFix:tryhook()
  end
end
InspectFix:SetScript("OnEvent", InspectFix_OnEvent)
InspectFix:RegisterEvent("ADDON_LOADED")

function InspectFix:Load()
  _G.NotifyInspect = NIhook
  InspectFix:tryhook()
  loaded = true
  print("InspectFix loaded.")
end

function InspectFix:Unload()
  _G.NotifyInspect = BlizzardNotifyInspect
  loaded = false
  print("InspectFix unloaded.")
end

InspectFix:Load()

SLASH_INSPECTFIX1 = "/inspectfix"
SLASH_INSPECTFIX2 = "/if"
SlashCmdList["INSPECTFIX"] = function(msg)
        local cmd = msg:lower()
        if cmd == "load" or cmd == "on" then
          InspectFix:Load()
        elseif cmd == "unload" or cmd == "off" then
          InspectFix:Unload()
        else
          print("/inspectfix [ on | off ]")
        end
end
